﻿using SQLite4Unity3d;
using UnityEngine;
#if !UNITY_EDITOR
using System.Collections;
using System.IO;
#endif
using System.Collections.Generic;

namespace U2DProjectCore { 
	/**
	@brief Contains logic for retrieving and creating SQLIte database data 

	Part of the Project Core Unity package.
	This script is based on a script from the SQLite4Unity package. It contains a 
	constructor that finds the local path to the database based on the hardware the
	game is currently running on. It also contains methods for creating each data
	table, methods for each individual querey used in the game, and methods for 
	adding new data entries into these tables. NOTE: every table requires its own
	non-Monobehavior-derived class using SQLite4Unity3d library.

	_Class output:_
	<br> Sends queries and construction information to SQLite.cs that accesses the
	SQLite database

	_Class input:_
	<br> ScoreController - reads and writes scores
	<br> Any class that stores or reads from database

	@author Jonny Aiello
	@author Krueger Systems, Inc.
	@version 0.1.1
	@date 4/27/16
	*/ 
	public class DataService  {

		private SQLiteConnection _connection;

		// [[ ----- CONSTRUCTOR ----- ]]
		public DataService(string DatabaseName){

	#if UNITY_EDITOR
	        var dbPath = string.Format(
	        	@"Assets/StreamingAssets/{0}", DatabaseName);
	#else
	        // check if file exists in Application.persistentDataPath
	        var filepath = string.Format(
	        	"{0}/{1}", 
	        	Application.persistentDataPath, 
	        	DatabaseName
	        	);

	        if (!File.Exists(filepath))
	        {
	            Debug.Log("Database not in Persistent path");
	            // if it doesn't ->
	            // open StreamingAssets directory and load the db ->

	#if UNITY_ANDROID 
				// this is the path to your StreamingAssets in android
	            var loadDb = new WWW(
	            "jar:file://" + Application.dataPath + "!/assets/" + DatabaseName);  
	           	
	           	// CAREFUL here, for safety reasons you shouldn't let this while 
	           	//loop unattended, place a timer and error check
	            while (!loadDb.isDone) { }  
	           
	            // then save to Application.persistentDataPath
	            File.WriteAllBytes(filepath, loadDb.bytes);

	#elif UNITY_IOS
				// this is the path to your StreamingAssets in iOS
	            var loadDb = Application.dataPath + "/Raw/" + DatabaseName;  
	            // then save to Application.persistentDataPath
	            File.Copy(loadDb, filepath);

	#elif UNITY_WP8
	            // this is the path to your StreamingAssets in Web Player
	            var loadDb = 
	            	Application.dataPath + "/StreamingAssets/" + DatabaseName;  
	            // then save to Application.persistentDataPath
	            File.Copy(loadDb, filepath);

	#elif UNITY_WINRT
				// this is the path to your StreamingAssets in iOS
				var loadDb = 
					Application.dataPath + "/StreamingAssets/" + DatabaseName;  
				// then save to Application.persistentDataPath
				File.Copy(loadDb, filepath);

	#else
				// this is the path to your StreamingAssets in iOS
				var loadDb = 
					Application.dataPath + "/StreamingAssets/" + DatabaseName;  
				// then save to Application.persistentDataPath
				File.Copy(loadDb, filepath);

	#endif

	            Debug.Log("Database written");
	        }

	        var dbPath = filepath;
	#endif
	        _connection = new SQLiteConnection(
	        	dbPath, SQLiteOpenFlags.ReadWrite | SQLiteOpenFlags.Create
	        	);
	        Debug.Log("Final PATH: " + dbPath);     
		}

	// -----------------------------------------------------------------------------
	// Score Database Services

		// [[ ----- CREATE SCORE DB ----- ]]
		public void CreateScoreDB(){
			_connection.DropTable<Score> ();
			_connection.CreateTable<Score> ();

			_connection.InsertAll (new[]{
				new Score{
					Id = 1,
					Points = 0,
					Year = System.DateTime.Now.Year,
					Month = System.DateTime.Now.Month,
					Day = System.DateTime.Now.Day,
					Hour = System.DateTime.Now.Hour,
					Minute = System.DateTime.Now.Minute,
					Second = System.DateTime.Now.Second
				}
			});
		}

		// [[ ----- CREATE SCORE ----- ]]
		public Score CreateScore( int _score ){
			var s = new Score{
				Points = _score,
				Year = System.DateTime.Now.Year,
				Month = System.DateTime.Now.Month,
				Day = System.DateTime.Now.Day,
				Hour = System.DateTime.Now.Hour,
				Minute = System.DateTime.Now.Minute,
				Second = System.DateTime.Now.Second
			};
			_connection.Insert(s);
			return s;
		}

		// [[ ----- GET SCORES ----- ]]
		public IEnumerable<Score> GetScores(){
			return _connection.Table<Score>();
		}

		// [[ ----- GET BY SCORE ----- ]]
		// returns all scores that match the score argument
		public IEnumerable<Score> GetByScore( int _points ){
			return _connection.Table<Score>().Where(x => x.Points == _points);
		}

		// [[ ----- GET SINGLE BY SCORE ----- ]]
		// returns earliest-created Score object that matches criteria
		public Score GetScoreByScoreOne( int _points ){
			return _connection.Table<Score>().Where(x => x.Points == _points)
				.FirstOrDefault();
		}

		// [[ ----- GET SINGLE BY DATE TIME ----- ]]
		// returns all scores that match the date argument
		public Score GetSingleByDateTime( 
			int _y,
			int _mo,
			int _d,
			int _h,
			int _mi,
			int _s 
			){

			return _connection.Table<Score>().Where(x =>( x.Year == _y)
				&& ( x.Month == _mo)
				&& ( x.Day == _d)
				&& ( x.Hour == _h)
				&& ( x.Minute == _mi)
				&& ( x.Second == _s)
				).FirstOrDefault();
		}

		// [[ ----- DELETE SCORE ----- ]]
		public void DeleteScore( Score _score ){
			_connection.Delete( _score ); 
		}
	}
}